
import java.awt.*;
import javax.swing.*;


/************************************************************************************
 * <b>PURPOSE</b>
 * <ol>
 * <li>To provide a base for a subsequent Stack assignment</li>
 *<li>To explore the underlying mathematics of an important class of fractals defined by Strings</li>
 * </ol>
 * <b>DATE</b>:	2008 11 14
 * @author C. D'Arcy
 ************************************************************************************/


public class LSystems extends JFrame {

  /** The width of the JPanel */
  int width = 300;

  /** The height of the JPanel */
  int height = 300;

  /**
   * Constructs the current LSystems object
   */
  public LSystems() {
    super("LSystems");
    Cage cage = new Cage(width, height);

    getContentPane().add(cage);
    setBounds(0, 0, width, height);
    setVisible(true);
  }

  public static void main(String[] args) {
    LSystems lSystems = new LSystems();

    lSystems.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
  }

}


/**
 * This inner class provides the canvas onto which is drawn one fractal space-filling
 * curve at a time.  
 */
class Cage extends JPanel {

  /** the fractal object*/
  Monster monster;
	
  /** the width of the canvas */
  int width;

  /** the height of the canvas */
  int height;

  /** the <i>x</i>-value of the start of line to be drawn in screen coordinates*/
  int currX;

  /** the <i>y</i>-value of the start of line to be drawn  in screen coordinates*/
  int currY;

  /** the <i>x</i>-value of the end of line to be drawn  in screen coordinates*/
  int nextX;

  /** the <i>y</i>-value of the end of line to be drawn  in screen coordinates*/
  int nextY;

  /** the length (in pixels) of the length of the line to be drawn */
  int distance;

  /** the turn angle of the turtle in degrees */
  double delta;

  /** the current orientation of the turtle in degrees */
  double heading;

  /** the number of generations to compute */
  int n;

  /** The current String definition of the fractal. Updated for each generation */  
  String current;
	
  /**
   * Constructor accepts the dimensions of the Canvas and instantiates a fractal object.
   * @param width the width of the canvas
   * @param height the height of the canvas
   */
  public Cage(int width, int height) {
    super();
    this.width = width;
    this.height = height;

    // ICS4M: add your monsters, one at a time, here...

  }

  /**
   * Accepts a reference to the Graphics object for the JPanel and draws the fractal. The method instantiates a local Turtle object 
   * to assist with the drawing. 
   */
  public void paintComponent(Graphics g) {

    Graphics2D g2 = (Graphics2D) g;

    g2.setBackground(Color.WHITE);
    g2.clearRect(0, 0, width, height);


    // ICS4M: draw your monster here...
 
 }


/**
 * This class encapsulates the details of a fractal space-filling curve.  
 * */
class Monster {

  /** the initiator (axiom) of the fractal */
  String axiom;

  /** holds the characters for which productions exist */
  String keys;

  /** Holds the characters defining the fractals as a sequence of drawing commands for the Turtle */
  String definition;

  /** The array of productions (generators) that replace their respective key in each generation */
  String[] productions;

  /** The distance or length of each line segment(decreases with each generation). */
  int distance;

  /** The angle the heading of the turtle is modified by at each turn */ 
  double delta;

  /** the factor used to decrease the distance with each generation */
  int factor;

  /** The number of generations to propagate. */
  int n;

  public Monster(String axiom,
      String keys,
      String[] productions,
      int distance,
      double delta,
      int factor,
      int n) {
    this.axiom = axiom;
    this.keys = keys;
    this.productions = productions;
    this.distance = distance;
    this.delta = delta;
    this.factor = factor;
    this.n = n;
    definition = axiom;
		
    // ICS4M: generate a mature description here...

  
  
  }	

  /**
   * Utility method used to fetch the respective production associated with each key (character) 
   * @param ch the key
   * @return the associated production
   */
  private String getSegment(char ch) {
    int which = keys.indexOf(ch);

    if (which < 0) {
      return "" + ch;
    }
    return productions[which];
  }

  /**
   * Accessor method to return the String definition of the fractal.
   * @return the current String definition
   */
  public String getDefinition() {
    return definition;
  }

  /**
   * Accessor method to return the angle the Turtle is to change with each turn.
   * @return the angle the Turtle is to change with each turn
   */
  public double getDelta() {
    return delta;
  }

  /**
   * Accessor method to return the length of the line segment to be drawn.
   * @return the length of the line segment to be drawn
   */
  public int getDistance() {
    return distance;
  }

}
}


